import click
from revisit.db.sqlite.manager import DatabaseManager
from revisit.db.sqlite.repository import BookmarkRepository
from revisit.core.utils import parse_indices

@click.command()
@click.argument('indices', required=False)
def update(indices):
    """
    Update the saved bookmarks.
    Accepts space-separated list of indices, hyphenated range or both.
    If no indices provided, iterates through all bookmarks.
    """
    db_manager = DatabaseManager()
    repo = BookmarkRepository(db_manager)
    
    if indices:
        ids = list(parse_indices(indices))
        bookmarks = repo.get_by_ids(ids)
    else:
        bookmarks = repo.list_all()
        
    if not bookmarks:
        click.echo("No bookmarks found to update.")
        return
        
    for b in bookmarks:
        click.echo(f"\nUpdating bookmark {b.id}: {b.name}")
        b.url = click.prompt("  URL", default=b.url)
        b.name = click.prompt("  Name", default=b.name)
        tags_str = click.prompt("  Tags (comma-separated)", default=",".join(b.tags))
        b.tags = [t.strip() for t in tags_str.split(",")] if tags_str else []
        repo.update(b)
        click.echo("  Updated successfully.")



import click
from revisit.db.sqlite.manager import DatabaseManager
from revisit.db.sqlite.repository import BookmarkRepository
from revisit.core.utils import parse_indices

@click.command()
@click.argument('indices')
def delete(indices):
    """
    Delete bookmarks by index.
    Accepts space-separated list of indices, hyphenated range or both.
    """
    db_manager = DatabaseManager()
    repo = BookmarkRepository(db_manager)
    
    ids = list(parse_indices(indices))
    if not ids:
        click.echo("No valid indices provided.")
        return
        
    repo.delete(ids)
    click.echo(f"Deleted bookmarks with indices: {', '.join(map(str, ids))}")



import click
from revisit.domain.bookmark import Bookmark
from revisit.db.sqlite.manager import DatabaseManager
from revisit.db.sqlite.repository import BookmarkRepository

@click.command()
@click.option('--url', required=True, help='URL of the bookmark.')
@click.option('--name', required=True, help='Name of the bookmark.')
@click.option('--tags', help='Tags of the bookmark (comma-separated).')
def add(url, name, tags):
    """Add a bookmark"""
    db_manager = DatabaseManager()
    repo = BookmarkRepository(db_manager)
    
    tag_list = [t.strip() for t in tags.split(",")] if tags else []
    bookmark = Bookmark(url=url, name=name, tags=tag_list)
    
    repo.add(bookmark)
    click.echo(f"Successfully added bookmark: {name} ({url})")

if __name__ == '__main__':
    add()



import click
from revisit.db.sqlite.manager import DatabaseManager
from revisit.db.sqlite.repository import BookmarkRepository
from revisit.core.utils import parse_indices

@click.command(name="print")
@click.argument('indices', required=False)
def print_cmd(indices):
    """
    Show the saved bookmarks by its database index.
    Accepts space-separated list of indices (e.g. 5 6 23 4 110 45),
    hyphenated range (e.g. 100-200) or both (e.g. 1-3 7 9).
    If no arguments, all records are shown.
    """
    db_manager = DatabaseManager()
    repo = BookmarkRepository(db_manager)
    
    if indices:
        ids = list(parse_indices(indices))
        bookmarks = repo.get_by_ids(ids)
    else:
        bookmarks = repo.list_all()
    
    if not bookmarks:
        click.echo("No bookmarks found.")
        return
    
    for b in bookmarks:
        tags_str = f" [{', '.join(b.tags)}]" if b.tags else ""
        click.echo(f"{b.id:3}: {b.name} - {b.url}{tags_str}")



from dataclasses import dataclass, field
from datetime import datetime
from typing import List, Optional

@dataclass
class Bookmark:
    url: str
    name: str
    tags: List[str] = field(default_factory=list)
    id: Optional[int] = None
    created_at: datetime = field(default_factory=datetime.now)

    @classmethod
    def from_dict(cls, data: dict):
        tags = data.get("tags", "")
        if isinstance(tags, str):
            tags = [t.strip() for t in tags.split(",") if t.strip()]
        
        return cls(
            url=data["url"],
            name=data["name"],
            tags=tags,
            id=data.get("id"),
            created_at=datetime.fromisoformat(data["created_at"]) if data.get("created_at") else datetime.now()
        )

    def to_dict(self):
        return {
            "id": self.id,
            "url": self.url,
            "name": self.name,
            "tags": ",".join(self.tags),
            "created_at": self.created_at.isoformat()
        }



import click
import re
from revisit.domain.bookmark import Bookmark
from revisit.db.sqlite.manager import DatabaseManager
from revisit.db.sqlite.repository import BookmarkRepository

@click.command(name="import")
@click.argument('input_file', type=click.Path(exists=True))
def import_cmd(input_file):
    """Import bookmarks from HTML file in Netscape Bookmark format"""
    db_manager = DatabaseManager()
    repo = BookmarkRepository(db_manager)
    
    with open(input_file, "r", encoding="utf-8") as f:
        content = f.read()
        
    # A simple regex to find bookmarks. 
    # Real Netscape format can be nested with folders, but we'll start with flat list.
    pattern = re.compile(r'<A HREF="([^"]+)"(?:[^>]*ADD_DATE="([^"]*)")?(?:[^>]*TAGS="([^"]*)")?[^>]*>([^<]*)</A>', re.IGNORECASE)
    
    matches = pattern.findall(content)
    if not matches:
        click.echo("No bookmarks found in file.")
        return
        
    count = 0
    for url, date, tags_str, name in matches:
        tag_list = [t.strip() for t in tags_str.split(",")] if tags_str else []
        bookmark = Bookmark(url=url, name=name or url, tags=tag_list)
        # Note: we ignore date for simplicity in adding, using current date
        repo.add(bookmark)
        count += 1
        
    click.echo(f"Imported {count} bookmarks from {input_file}")



import click
import requests
from revisit.db.sqlite.manager import DatabaseManager
from revisit.db.sqlite.repository import BookmarkRepository

@click.command()
def check():
    """Check if links still exist on the internet"""
    db_manager = DatabaseManager()
    repo = BookmarkRepository(db_manager)
    bookmarks = repo.list_all()
    
    if not bookmarks:
        click.echo("No bookmarks to check.")
        return
        
    click.echo(f"Checking {len(bookmarks)} bookmarks...")
    for b in bookmarks:
        try:
            # Using a browser-like User-Agent to avoid some common blocks
            headers = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.124 Safari/537.36'}
            response = requests.head(b.url, timeout=10, headers=headers, allow_redirects=True)
            
            # If HEAD is not allowed, try GET but only first few bytes
            if response.status_code == 405:
                response = requests.get(b.url, timeout=10, headers=headers, stream=True)
                response.close()

            if response.status_code < 400:
                click.echo(click.style(f"  ✓ {b.id:3}: {b.name} is OK", fg="green"))
            else:
                click.echo(click.style(f"  ✗ {b.id:3}: {b.name} returned status {response.status_code}", fg="red"))
        except requests.RequestException as e:
            click.echo(click.style(f"  ✗ {b.id:3}: {b.name} failed: {type(e).__name__}", fg="red"))



import click
from flask import Flask, render_template_string
from revisit.db.sqlite.manager import DatabaseManager
from revisit.db.sqlite.repository import BookmarkRepository

def create_app():
    app = Flask(__name__)
    db_manager = DatabaseManager()
    repo = BookmarkRepository(db_manager)

    HTML_TEMPLATE = """
    <!DOCTYPE html>
    <html lang="en">
    <head>
        <meta charset="UTF-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0">
        <title>Revisit Bookmark Manager</title>
        <style>
            :root {
                --primary: #2563eb;
                --bg: #f8fafc;
                --text: #1e293b;
                --card-bg: #ffffff;
            }
            body { 
                font-family: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, Helvetica, Arial, sans-serif;
                margin: 0;
                background: var(--bg);
                color: var(--text);
                line-height: 1.5;
            }
            .container {
                max-width: 900px;
                margin: 40px auto;
                padding: 0 20px;
            }
            header {
                margin-bottom: 40px;
                display: flex;
                justify-content: space-between;
                align-items: center;
            }
            h1 { margin: 0; font-size: 2rem; color: var(--primary); }
            .bookmark-list {
                display: grid;
                gap: 16px;
            }
            .bookmark-card {
                background: var(--card-bg);
                padding: 20px;
                border-radius: 12px;
                box-shadow: 0 1px 3px rgba(0,0,0,0.1);
                transition: transform 0.2s, box-shadow 0.2s;
            }
            .bookmark-card:hover {
                transform: translateY(-2px);
                box-shadow: 0 4px 6px -1px rgba(0,0,0,0.1), 0 2px 4px -1px rgba(0,0,0,0.06);
            }
            .bookmark-name {
                display: block;
                font-size: 1.25rem;
                font-weight: 600;
                color: var(--primary);
                text-decoration: none;
                margin-bottom: 4px;
            }
            .bookmark-url {
                font-size: 0.875rem;
                color: #64748b;
                word-break: break-all;
            }
            .tags {
                margin-top: 12px;
                display: flex;
                flex-wrap: wrap;
                gap: 8px;
            }
            .tag {
                background: #f1f5f9;
                color: #475569;
                padding: 4px 10px;
                border-radius: 9999px;
                font-size: 0.75rem;
                font-weight: 500;
            }
            .empty-state {
                text-align: center;
                padding: 60px;
                color: #64748b;
            }
        </style>
    </head>
    <body>
        <div class="container">
            <header>
                <h1>Revisit</h1>
                <span>{{ bookmarks|length }} Bookmarks</span>
            </header>
            
            <div class="bookmark-list">
                {% for b in bookmarks %}
                <div class="bookmark-card">
                    <a href="{{ b.url }}" class="bookmark-name" target="_blank">{{ b.name }}</a>
                    <div class="bookmark-url">{{ b.url }}</div>
                    {% if b.tags %}
                    <div class="tags">
                        {% for tag in b.tags %}
                        <span class="tag">{{ tag }}</span>
                        {% endfor %}
                    </div>
                    {% endif %}
                </div>
                {% else %}
                <div class="empty-state">
                    <p>No bookmarks found. Use <code>revisit add</code> to get started!</p>
                </div>
                {% endfor %}
            </div>
        </div>
    </body>
    </html>
    """

    @app.route('/')
    def index():
        bookmarks = repo.list_all()
        return render_template_string(HTML_TEMPLATE, bookmarks=bookmarks)

    return app

@click.command()
@click.option('--port', default=8080, help='Port to run the server on.')
def server(port):
    """Run a simple and performant web server which serves the site for managing bookmarks"""
    app = create_app()
    click.echo(f"Starting revisit server on http://localhost:{port}")
    app.run(port=port, host='0.0.0.0')



def parse_indices(index_str: str) -> set[int]:
    """
    Parses a string of space-separated indices and hyphenated ranges.
    Example: "1-3 7 9" -> {1, 2, 3, 7, 9}
    """
    indices = set()
    parts = index_str.split()
    for part in parts:
        if "-" in part:
            try:
                start_str, end_str = part.split("-")
                start, end = int(start_str), int(end_str)
                for i in range(start, end + 1):
                    indices.add(i)
            except (ValueError, IndexError):
                continue
        else:
            try:
                indices.add(int(part))
            except ValueError:
                continue
    return indices



from typing import List, Optional
from revisit.domain.bookmark import Bookmark
from revisit.db.sqlite.manager import DatabaseManager

class BookmarkRepository:
    def __init__(self, db_manager: DatabaseManager):
        self.db_manager = db_manager

    def add(self, bookmark: Bookmark) -> Bookmark:
        query = """
        INSERT INTO bookmarks (url, name, tags, created_at)
        VALUES (?, ?, ?, ?)
        """
        with self.db_manager.get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute(query, (
                bookmark.url,
                bookmark.name,
                ",".join(bookmark.tags),
                bookmark.created_at.isoformat()
            ))
            bookmark.id = cursor.lastrowid
            conn.commit()
        return bookmark

    def list_all(self) -> List[Bookmark]:
        query = "SELECT * FROM bookmarks ORDER BY id ASC"
        bookmarks = []
        with self.db_manager.get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute(query)
            rows = cursor.fetchall()
            for row in rows:
                bookmarks.append(Bookmark.from_dict(dict(row)))
        return bookmarks

    def get_by_ids(self, ids: List[int]) -> List[Bookmark]:
        if not ids:
            return []
        placeholders = ",".join(["?"] * len(ids))
        query = f"SELECT * FROM bookmarks WHERE id IN ({placeholders}) ORDER BY id ASC"
        bookmarks = []
        with self.db_manager.get_connection() as conn:
            cursor = conn.cursor()
            cursor.execute(query, ids)
            rows = cursor.fetchall()
            for row in rows:
                bookmarks.append(Bookmark.from_dict(dict(row)))
        return bookmarks

    def delete(self, ids: List[int]):
        if not ids:
            return
        placeholders = ",".join(["?"] * len(ids))
        query = f"DELETE FROM bookmarks WHERE id IN ({placeholders})"
        with self.db_manager.get_connection() as conn:
            conn.execute(query, ids)
            conn.commit()

    def update(self, bookmark: Bookmark):
        query = """
        UPDATE bookmarks 
        SET url = ?, name = ?, tags = ?
        WHERE id = ?
        """
        with self.db_manager.get_connection() as conn:
            conn.execute(query, (
                bookmark.url,
                bookmark.name,
                ",".join(bookmark.tags),
                bookmark.id
            ))
            conn.commit()



import click
import time
from revisit.db.sqlite.manager import DatabaseManager
from revisit.db.sqlite.repository import BookmarkRepository

@click.command()
@click.argument('output_file', type=click.Path())
def export(output_file):
    """Export bookmarks into HTML file in Netscape Bookmark format"""
    db_manager = DatabaseManager()
    repo = BookmarkRepository(db_manager)
    bookmarks = repo.list_all()
    
    if not bookmarks:
        click.echo("No bookmarks to export.")
        return
        
    with open(output_file, "w", encoding="utf-8") as f:
        f.write("<!DOCTYPE NETSCAPE-Bookmark-file-1>\n")
        f.write("<!-- This is an automatically generated file.\n")
        f.write("     It will be read and overwritten.\n")
        f.write("     DO NOT EDIT! -->\n")
        f.write('<META HTTP-EQUIV="Content-Type" CONTENT="text/html; charset=UTF-8">\n')
        f.write("<TITLE>Bookmarks</TITLE>\n")
        f.write("<H1>Bookmarks</H1>\n")
        f.write("<DL><p>\n")
        
        for b in bookmarks:
            # Convert datetime to unix timestamp
            timestamp = int(b.created_at.timestamp())
            tags = ",".join(b.tags)
            f.write(f'    <DT><A HREF="{b.url}" ADD_DATE="{timestamp}" TAGS="{tags}">{b.name}</A>\n')
            
        f.write("</DL><p>\n")
        
    click.echo(f"Exported {len(bookmarks)} bookmarks to {output_file}")



import click
import webbrowser
from revisit.db.sqlite.manager import DatabaseManager
from revisit.db.sqlite.repository import BookmarkRepository
from revisit.core.utils import parse_indices

@click.command(name="open")
@click.argument('indices', required=False)
def open_cmd(indices):
    """
    Open bookmarks in browser.
    Accepts space-separated list of indices, hyphenated range or both.
    If no arguments, asks for ID.
    """
    db_manager = DatabaseManager()
    repo = BookmarkRepository(db_manager)
    
    if not indices:
        indices = click.prompt("Enter bookmark ID(s) to open")
        
    ids = list(parse_indices(indices))
    if not ids:
        click.echo("No valid indices provided.")
        return

    bookmarks = repo.get_by_ids(ids)
    
    if not bookmarks:
        click.echo("No bookmarks found for given indices.")
        return
        
    for b in bookmarks:
        click.echo(f"Opening: {b.name} ({b.url})")
        webbrowser.open(b.url)



import pytest
from click.testing import CliRunner
from revisit.cli import revisit

def test_version_command():
    runner = CliRunner()
    result = runner.invoke(revisit, ['version'])
    assert result.exit_code == 0
    assert 'revisit version' in result.output

def test_print_empty():
    # Using a separate DB for testing would be better, 
    # but for now we just check if it runs without error.
    runner = CliRunner()
    result = runner.invoke(revisit, ['print'])
    # Might exit with 0 even if no bookmarks
    assert result.exit_code == 0



import click
from revisit.commands.add_command import add
from revisit.commands.version_command import version
from revisit.commands.print_command import print_cmd
from revisit.commands.delete_command import delete
from revisit.commands.update_command import update
from revisit.commands.open_command import open_cmd
from revisit.commands.check_command import check
from revisit.commands.export_command import export
from revisit.commands.import_command import import_cmd
from revisit.commands.server_command import server

@click.group()
def revisit():
    """Revisit is a bookmark manager with custom webview"""
    pass

revisit.add_command(add)
revisit.add_command(version)
revisit.add_command(print_cmd)
revisit.add_command(delete)
revisit.add_command(update)
revisit.add_command(open_cmd)
revisit.add_command(check)
revisit.add_command(export)
revisit.add_command(import_cmd)
revisit.add_command(server)

if __name__ == '__main__':
    revisit()



import sqlite3
import os
from pathlib import Path

class DatabaseManager:
    def __init__(self, db_path: str = "bookmarks.db"):
        self.db_path = db_path
        self._init_db()

    def _init_db(self):
        """Initialize the database and run migrations."""
        if not os.path.exists(self.db_path):
            Path(self.db_path).touch()
        
        self.run_migrations()

    def get_connection(self):
        conn = sqlite3.connect(self.db_path)
        conn.row_factory = sqlite3.Row
        return conn

    def run_migrations(self):
        migrations_dir = Path(__file__).parent / "migrations"
        if not migrations_dir.exists():
            raise FileNotFoundError(f"Migrations directory not found at {migrations_dir}. "
                                  "Ensure the package is installed correctly with all data files.")

        with self.get_connection() as conn:
            # We could implement a migrations table to track applied migrations,
            # but for this simple version, we'll just run them if table doesn't exist.
            # The SQL itself has IF NOT EXISTS.
            for migration_file in sorted(migrations_dir.glob("*.sql")):
                with open(migration_file, "r") as f:
                    conn.executescript(f.read())
            conn.commit()
